package com.vincenzomanzoni.barbiere;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JButton;

public class AreaTaglio
{
    private JScrollPane jsp;
    private JTextArea area;

    private static final int NUMBARBIERI = 3;

    private Barbiere[]       barb; // i barbieri
    private JButton[]     bottone; // i bottoni che determinano la fine del taglio
    private boolean[]    occupata; // occupata da un CLIENTE
    private String[]    chiseduto; // "Cliente " o "Barbiere " + id (il barbiere ci dorme)
    private boolean[]   clipronto; // il cliente sulla sedia i e' pronto e aspetta il taglio
    private boolean[] tagliofatto; // Il taglio  completo (a giudizio del BARBIERE)
    private boolean[]   clifelice; // cliente sulla i-esima sedia soddisfatto del taglio

    public AreaTaglio(JTextArea a, JScrollPane sp)
    {
      area = a;
      jsp = sp;
      barb        = new Barbiere[NUMBARBIERI];
      bottone     = new JButton[NUMBARBIERI];
      occupata    = new boolean[NUMBARBIERI];
      chiseduto   = new String[NUMBARBIERI];
      clipronto   = new boolean[NUMBARBIERI];
      tagliofatto = new boolean[NUMBARBIERI];
      clifelice   = new boolean[NUMBARBIERI];
      int i=0;
      for (i=0;i<NUMBARBIERI;i++) {
        barb[i]      = null;
        bottone[i]   = null;
        occupata[i]  = false;
        chiseduto[i] = "Barbiere " + (i+1);
        clipronto[i] = false;
        tagliofatto[i] = false;
        clifelice[i] = false;
      }
    }

    /************************
    * Metodi per il Cliente *
    * **********************/

    public synchronized void vadoAlTaglio() {  //Cliente
      int sedia = primaSediaLibera();
      while ( sedia == -1) {
        print("Nessuna sedia libera - continuo a leggere");
        try { wait(); } catch (InterruptedException e) { }
        sedia = primaSediaLibera();
      }
      ((Cliente)Thread.currentThread()).atrio.miAlzoDalDivano();
      occupata[sedia] = true;
      chiseduto[sedia] = Thread.currentThread().getName(); //barb[sedia] in piedi!
      print("Ora scelgo il taglio...");
      clipronto[sedia] = true;
      notifyAll();
    }

    public synchronized void sonoSoddisfatto(int i) {  //(Cliente) lo fa il bottone per lui
      print("       ***       PRESSIONE DEL TASTO " + i + "         ***");
      area.append("Cliente " + i + " : E' un taglio perfetto. Grazie, Barbiere " + i + " !\n");
      jsp.getVerticalScrollBar().setValue(jsp.getVerticalScrollBar().getMaximum());
      tagliofatto[i-1] = true;
      clifelice[i-1] = true;
      bottone[i-1].setEnabled(false);
      notifyAll();
    }

    /*************************
    * Metodi per il Barbiere *
    * ***********************/

    public synchronized void inizioTaglio() {  //Barbiere
      int sedia = sediaACuiLavoro();
      while (!clipronto[sedia]) {
        try { wait(); } catch (InterruptedException e) { }
      }
      print("Taglio i capelli a " + chiseduto[sedia]);
      bottone[sedia].setEnabled(true);
      notifyAll();
    }

    public synchronized void congedoCliente() {  //Barbiere
      int sedia = sediaACuiLavoro();
      while (!clifelice[sedia]) {
        try { wait(); } catch (InterruptedException e) { }
      }
      print("Congedo e ringrazio il " + chiseduto[sedia]);
      clifelice[sedia] = false;
      tagliofatto[sedia] = false;
      clipronto[sedia] = false;
      chiseduto[sedia] = Thread.currentThread().getName();
      occupata[sedia] = false;
      notifyAll();
    }

    /*********************
    * Metodi di servizio *
    * *******************/

    public synchronized String stato() {
      String status = "";
      int i=0;
        status += " Sedia  ChiSeduto\tOccupata\tCliPronto\tTagliato\tSoddisfatto\n";
      for(i=0;i<NUMBARBIERI;i++) {
        status += "   " + (i+1) + "    " + chiseduto[i] +  "\t  " + occupata[i] + "\t  ";
        status += clipronto[i] + "\t  " + tagliofatto[i] + "\t  " + clifelice[i] + "\n";
      }
      return status;
    }

    /* Associa biunivocamente il barbiere alla sedia */
    public void associaBottoneBarbiere(int i, JButton b) {
      barb[i-1] = new Barbiere(i, this);
      bottone[i-1] = b;
      barb[i-1].start();
      area.append("Inizia a lavorare il Barbiere " + i + "\n");
    }

    private int sediaACuiLavoro() { //Barbiere
      int sedia=-1, i=0;
      for(i=0;i<NUMBARBIERI;i++)
        if (Thread.currentThread() == barb[i]) sedia = i;
      if(sedia==-1) print("ERRORE - non trovo la sedia a cui lavoro");
      return sedia;
    }

    private int sediaCheOccupo() { //Cliente
      int miasedia=-1, i=0;
      for(i=0;i<NUMBARBIERI;i++)
        if (Thread.currentThread().getName().equals(chiseduto[i])) miasedia = i;
      if(miasedia==-1) print("ERRORE - non trovo la sedia su cui siedo");
      return miasedia;
    }

    private synchronized int primaSediaLibera() {
      int i=0;
      for (i=0;i<NUMBARBIERI;i++)
        if(!occupata[i]) return i;
      return -1;
    }

    private void print(String msg) {
        String name = Thread.currentThread().getName();
        area.append(name + ": " + msg + "\n");
        jsp.getVerticalScrollBar().setValue(jsp.getVerticalScrollBar().getMaximum());
    }
}
